/****************************************************************************
* FILE : lex_anal.l
* INPUT/OUTPUT  : lex input file for PascalD to PostScript project
* PROJECT: CS 541 CS-University of Kentucky
* SOURCE:   Provided as a handout by Jerzy W. Jaromczyk
* COPYRIGHT: University of Kentucky @1999
****************************************************************************/


%{

  #include <stdlib.h>
  #include <stdio.h>
  #include <string.h>
  #include <errno.h>
  #include "rddefs.h"

#define YY_NEVER_INTERACTIVE 1

  /* rd++ grammar symbol values structure. */
  extern rdunion rdlval;
  /* line count defined in rdparser.cc */
  extern int rdlines;
  /* look ahead */
  int lookahead;
  /* Identifier tracking. */
  char *id_lookback;
  /* Overflow flag set to errno from atoi and atof. */
  int overflow;
  /* global routines defined in this file. */
  void InitLex(void);


%}

/* Regular Expressions */
delim          ([ \t])
ws             ({delim}+)
letter         ([A-Za-z])
digit          ([0-9])
id             ({letter}({letter}|{digit}|_)*)
int_num        ({digit}+)
real_num       ({digit}+\.{digit}+(E[+\-]?{digit}+)?)

/* Exclusive start state for comment mode. */
%x CMNT_MODE VERBATIM_CODE

%%
<<EOF>>        { fprintf(stderr, "DOJ:EOF\n"); yyterminate(); }

"/*"                    BEGIN CMNT_MODE;
"(*"                    BEGIN CMNT_MODE;
<CMNT_MODE>.            {;}
<CMNT_MODE>\n           {rdlines++;}
<CMNT_MODE>"*/"         BEGIN INITIAL;     /* Return to Regular Processing */
<CMNT_MODE>"*)"         BEGIN INITIAL;     /* Return to Regular Processing */

"{#"                    BEGIN VERBATIM_CODE;
<VERBATIM_CODE>.        {
                           printf(yytext);
                        }
<VERBATIM_CODE>\n       {
                           printf("\n");
			   rdlines++;
                        }
<VERBATIM_CODE>"#}"     BEGIN INITIAL;

{ws}+            {;}
[\n]             {
		  rdlines++;
	         }
"if"             { return lookahead = IF; }
"then"           { return lookahead = THEN; }
"else"           { return lookahead = ELSE; }
"while"          { return lookahead = WHILE; }
"do"             { return lookahead = DO; }
"begin"          { return lookahead = BEGIN_BLOCK; }
"end"            { return lookahead = END_BLOCK; }
"read"           { return lookahead = READ; }
"write"          { return lookahead = WRITE; }
"writeln"        { return lookahead = WRITELN; }
"program"        { return lookahead = PROGRAM; }
"var"            { return lookahead = VAR; }
"procedure"      { return lookahead = PROCEDURE; }
"function"       { return lookahead = FUNCTION; }
"integer"        { return lookahead = INTEGER; }
"real"           { return lookahead = REAL; }
"array"          { return lookahead = ARRAY; }
"of"             { return lookahead = OF; }
"for"		 { return lookahead = FOR;}
"to"		 { return lookahead = TO;}
{real_num}     {
		    rdlval.Float = (double)atof(yytext);
                    overflow = errno;
	            return lookahead = REAL_NUM;
	       }
{int_num}      {
                    rdlval.Integer = atoi(yytext);
		    overflow = errno;
                    return lookahead = INT_NUM;
	       }
"<"            {
                  rdlval.Op = LT;
                  return lookahead = RELOP;
               }
"<="           {
                  rdlval.Op = LE;
                  return lookahead = RELOP;
               }
"="            {
                  rdlval.Op = EQ;
                  return lookahead = RELOP;
               }
"<>"           {
                  rdlval.Op = NE;
                  return lookahead = RELOP;
               }
">"            {
                  rdlval.Op = GT;
                  return lookahead = RELOP;
               }
">="           {
                  rdlval.Op = GE;
                  return lookahead = RELOP;
               }
"+"            {
                  rdlval.Op = PLUS;
                  if(lookahead == RBRACK || lookahead == RPAREN ||
                     lookahead == INT_NUM || lookahead == REAL_NUM ||
                     lookahead == ID) {
                            return lookahead = ADDOP;
                  } else {
                      return lookahead = SIGN;
                  }
               }
"-"            {
                  rdlval.Op = MINUS;
                  if(lookahead == RBRACK || lookahead == RPAREN ||
                  lookahead == INT_NUM || lookahead == REAL_NUM ||
                  lookahead == ID) {
                      return lookahead = ADDOP;
                  } else {
                      return lookahead = SIGN;
                  }
               }
"or"             {
                  rdlval.Op = OR;
                  return lookahead = ADDOP;
               }
"div"            {
                  rdlval.Op = DIV;
                  return lookahead = MULOP;
               }
"mod"            {
		  rdlval.Op = MOD;
	          return lookahead = MULOP;
	       }
"/"            {
		  rdlval.Op = DIVIDE;
	          return lookahead = MULOP;
	       }
"*"            {
		  rdlval.Op = MULTIPLY;
	          return lookahead = MULOP;
	       }
"and"          {
		  rdlval.Op = AND;
                  return lookahead = MULOP;
	       }
":="           { return lookahead = ASSIGN; }
"not"          { return lookahead = NOT; }
"["            { return lookahead = LBRACK; }
"]"            { return lookahead = RBRACK; }
"("            { return lookahead = LPAREN; }
")"            { return lookahead = RPAREN; }
":"            { return lookahead = COLON; }
";"            { return lookahead = SEMICOLON; }
".."           { return lookahead = RANGE; }
"."            { yyterminate(); return lookahead = PERIOD; }
","            { return lookahead = COMMA; }
{id}           {
                 /*fprintf(stderr,"Identifier: %s len: %i\n",yytext); */
		 id_lookback = (char *)malloc(yyleng);
		 memcpy(id_lookback, yytext, yyleng + 1);
                 return lookahead = ID;
               }
.              { return lookahead = UNKNOWN; }

%%
/* End of Patterns */

int yywrap() {return 0;}

/* InitLex() */
void InitLex(void)
{
  rdlines = 1;
  id_lookback = NULL;
  /* setup the input stream for lex */
  yyin = stdin;
  /* setup the output stream for lex */
  yyout = stderr;
}
