/*
  sym_tabl.h
*/

#ifndef _SYMBOLTABLE_H_
#define _SYMBOLTABLE_H_

#define MAX_SYMBOL_TABLES 31
#define SYMBOL_TABLE_SIZE 293
#define SPELLING_TABLE_SIZE 2024
#define MAX_TOKEN_LENGTH 20

/* Type Definitions */
#define T_FUNCTION 5
#define T_PROCEDURE 15
#define T_VAR 35
#define T_ARRAY 45
#define T_RECORD 55
#define T_TYPE 65

enum BOOL {FALSE, TRUE};
#include <stdlib.h>
#include <stdio.h>

extern int lookahead;
extern int compile_errors;
extern int rdlines;

// Functions to compute the hash value
// and insert a symbol into the hash table
#define hash(s) token_hash(s, SYMBOL_TABLE_SIZE)

class LL_Node
{
   protected:
        char *spelling_start;
        char *name;
        int line_number;
  /*  begin_line is the first line(ie the line to give to BSR). */
        int begin_line;
  /* offset for a variable is the offset in the activation record. */
	int offset;
  /* block_level is the block level at which a symbol occurs. */
        int block_level;
        LL_Node *next;
  /* One of PROCEDURE, VAR, ARRAY, RECORD, TYPE. */
        int definition_type;
  /* An ARRAY will have a dimension value. */
        int dimension;
        int size;
        char *base_name;
  /* This is for Records and Procedures. */
  /* For T_RECORD it is a list of Fields. */
  /* For T_PROCEDURE it is a list of Parameters. */
  /* For T_INTEGER it is a pointer to the predefined symbol for integer. */
  /* For T_USER it is the type definition symbol. */
  /* For T_ARRAY it is the type definition symbol. */
  /* For T_DEFINED it is NULL */
        void *symbol_list;
   public:
        LL_Node();
        ~LL_Node();
        void SetNext(LL_Node *new_next) {next = new_next;};
        LL_Node *Next(void) {return next;}
        void SetSpelling(char *start);
        char *GetSpelling(void) {return spelling_start;}
        char *GetName(void);
        int GetLineNumber(void) {return line_number;}
        void SetLineNumber(int lnum) {line_number = lnum;}
  /* Definition Functions */
        int GetDefinitionType(void) {return definition_type;}
        void SetDefinitionType(int dtype) {definition_type = dtype;}
        int GetDimension(void) {return dimension;}
        void SetDimension(int new_d) {dimension = new_d;}
        char *GetBaseName(void) {return base_name;}
        void SetBaseName(char *b_name) {base_name = b_name;}
  /* Need to set up parameters for records, and procedures... */
        void SetSymbolList(void *new_list) { symbol_list = new_list; }
        void *GetSymbolList() { return symbol_list; }
	int GetOffset() { return offset;}
        void SetBlockLevel(int blklevel) { block_level = blklevel; }
        int  GetBlockLevel() { return block_level; }
	void SetOffset(int);
        int  GetSize() { return size;}
	void SetSize(int new_size) { size = new_size; }
        void SetBeginLine(int new_bl) { begin_line = new_bl; }
        int  GetBeginLine() { return begin_line; }

  /* Recursively descend the symbol_lists of an LL_Node, printing the contents. */
        void FPrintF();
};

class L_List
{
   protected:
     LL_Node *head;
     LL_Node *last;
     LL_Node *current_pos;
     int size, pos_num;

   public:
     L_List();		   // Linked List Constructor
    ~L_List();		   // Linked List Destructor
    void Clear(void);  // Link list deletion function, frees memory.
    void First(void);  // Make current_pos the first node in the list.
    void Last(void);   // Make current_pos the last node in the list.
    void Next(void);   // Make current_pos the node following the current node.
    // Insert a symbol at the head of a list
    // Returns current_pos after insert(ie the head);
    void Insert(char *start);
    LL_Node *Lookup(char *token);                      // Search a list for a symbol.
    LL_Node *GetCurrent() { return current_pos; }  // returns the current symbol
    int GetSize(void) { return size; }
    int GetPosNum(void) { return pos_num; }

};

class Symbol_Table
{
   protected:
        L_List SymbolTable[SYMBOL_TABLE_SIZE];
        char Spelling_Table[SPELLING_TABLE_SIZE];
        int table_usage;
        char *lastchar;
	BOOL declaration;  // Is the current token in a declaration?
        char *lookup_start;

   public:
	BOOL globldeclaration;// used to determine if this line of code is
			      // the declarations line
	BOOL globalbegin;    // used to determine if the identifier is inside
			     // the body of the Procedure or Module
        Symbol_Table();
        ~Symbol_Table();
        LL_Node *Insert(char *token);
        LL_Node *Lookup(char *token, int hash_value);

        void FPrintF(FILE *outstream);
        void QueryMode(void);
};

#endif




